@@ -2,29 +2,22 @@
 calphy: a Python library and command line interface for automated free
 energy calculations.
 
-Copyright 2021  (c) Sarath Menon^1, Yury Lysogorskiy^1, Ralf Drautz^1
-^1: Ruhr-University Bochum, Bochum, Germany
+Copyright 2021  (c) Sarath Menon^1, Yury Lysogorskiy^2, Ralf Drautz^2
+^1: Max Planck Institut für Eisenforschung, Dusseldorf, Germany 
+^2: Ruhr-University Bochum, Bochum, Germany
+
+calphy is published and distributed under the Academic Software License v1.0 (ASL). 
+calphy is distributed in the hope that it will be useful for non-commercial academic research, 
+but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+See the ASL for more details. 
 
 More information about the program can be found in:
-Menon, Sarath, Yury Lysogorskiy, Jutta Rogal, and Ralf Drautz. 
-“Automated Free Energy Calculation from Atomistic Simulations.” 
-ArXiv:2107.08980 [Cond-Mat], July 19, 2021. 
-http://arxiv.org/abs/2107.08980.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-See the LICENSE file.
+Menon, Sarath, Yury Lysogorskiy, Jutta Rogal, and Ralf Drautz.
+“Automated Free Energy Calculation from Atomistic Simulations.” Physical Review Materials 5(10), 2021
+DOI: 10.1103/PhysRevMaterials.5.103801
 
 For more information contact:
-sarath.menon@ruhr-uni-bochum.de
+sarath.menon@ruhr-uni-bochum.de/yury.lysogorskiy@icams.rub.de
 """
 
 import numpy as np
@@ -601,3 +594,58 @@ def integrate_rs(simfolder, f0, t, natoms, p=0, nsims=5,
         np.savetxt(outfile, np.column_stack((temp, f, werr)))
     else:
         return (temp, f, werr)
+
+
+def integrate_ps(simfolder, f0, natoms, pi, pf, nsims=1, 
+    return_values=False):
+    """
+    Carry out the reversible scaling integration
+    
+    Parameters
+    ----------
+    simfolder : string
+        main simulation folder
+    f0 : float
+        initial free energy for integration
+    nsims : int, optional
+        number of independent switching
+    
+    Returns
+    -------
+    None
+    
+    Notes
+    -----
+    Writes the output in a file pressure_sweep.dat
+    
+    """
+
+    ws = []
+
+    for i in range(1, nsims+1):
+        _, fp, fvol, _ = np.loadtxt(os.path.join(simfolder, "forward_%d.dat"%i), unpack=True, comments="#")
+        _, bp, bvol, _ = np.loadtxt(os.path.join(simfolder, "backward_%d.dat"%i), unpack=True, comments="#")
+        
+        fvol = fvol/natoms
+        bvol = bvol/natoms
+        
+        fp = fp/(10000*160.21766208)
+        bp = bp/(10000*160.21766208)
+
+        wf = cumtrapz(fvol, fp, initial=0)
+        wb = cumtrapz(bvol[::-1], bp[::-1], initial=0)
+
+        w = (wf + wb)/2
+        ws.append(w)
+
+    wmean = np.mean(ws, axis=0)
+    werr = np.std(ws, axis=0)
+    press = fp*(10000*160.21766208)
+
+    f = f0 + wmean
+
+    if not return_values:
+        outfile = os.path.join(simfolder, "pressure_sweep.dat")
+        np.savetxt(outfile, np.column_stack((press, f, werr)))
+    else:
+        return (press, f, werr)
\ No newline at end of file
