@@ -2,41 +2,37 @@
 calphy: a Python library and command line interface for automated free
 energy calculations.
 
-Copyright 2021  (c) Sarath Menon^1, Yury Lysogorskiy^1, Ralf Drautz^1
-^1: Ruhr-University Bochum, Bochum, Germany
+Copyright 2021  (c) Sarath Menon^1, Yury Lysogorskiy^2, Ralf Drautz^2
+^1: Max Planck Institut für Eisenforschung, Dusseldorf, Germany 
+^2: Ruhr-University Bochum, Bochum, Germany
+
+calphy is published and distributed under the Academic Software License v1.0 (ASL). 
+calphy is distributed in the hope that it will be useful for non-commercial academic research, 
+but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+See the ASL for more details. 
 
 More information about the program can be found in:
-Menon, Sarath, Yury Lysogorskiy, Jutta Rogal, and Ralf Drautz. 
-“Automated Free Energy Calculation from Atomistic Simulations.” 
-ArXiv:2107.08980 [Cond-Mat], July 19, 2021. 
-http://arxiv.org/abs/2107.08980.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-See the LICENSE file.
+Menon, Sarath, Yury Lysogorskiy, Jutta Rogal, and Ralf Drautz.
+“Automated Free Energy Calculation from Atomistic Simulations.” Physical Review Materials 5(10), 2021
+DOI: 10.1103/PhysRevMaterials.5.103801
 
 For more information contact:
-sarath.menon@ruhr-uni-bochum.de
+sarath.menon@ruhr-uni-bochum.de/yury.lysogorskiy@icams.rub.de
 """
 
 from mendeleev import element
 import copy
 import numpy as np
 import os
+import time
 
-from calphy.input import read_inputfile, create_identifier
-import calphy.queuekernel as cq
+from calphy.input import read_inputfile
+#import calphy.queuekernel as cq
 from calphy.errors import *
 import calphy.helpers as ph
 
+from calphy.liquid import Liquid
+from calphy.solid import Solid
 
 class MeltingTemp:
     """
@@ -54,18 +50,17 @@ class MeltingTemp:
     simfolder : string
         base folder for running calculations
     """
-    def __init__(self, options=None, kernel=None, simfolder=None):
-        self.options = options
-        self.kernel = kernel
+    def __init__(self, calculation=None, simfolder=None):
+        self.calc = calculation
         self.simfolder = simfolder
         self.org_tm = 0
-        self.calc = self.options['calculations'][kernel]
-        self.dtemp = self.calc['dtemp']
-        self.maxattempts = self.calc['maxattempts']
+        self.dtemp = self.calc.melting_temperature.step
+        self.maxattempts = self.calc.melting_temperature.attempts
         self.attempts = 0
-        self.exp_tm = self.calc['tguess']
+        self.exp_tm = self.calc._temperature
+        self.calculations = []
 
-        self.get_props(self.calc['element'][0])
+        self.get_props(self.calc.element[0])
         self.get_trange()
         self.arg = None
         
@@ -85,25 +80,25 @@ class MeltingTemp:
         -------
         None 
         """
-        self.calc['temperature'] = int(self.tmin)
-        self.calc['temperature_stop'] = int(self.tmax)
+        self.calc.temperature = [int(self.tmin), int(self.tmax)]
+        self.calc._temperature_stop = int(self.tmax)
         csol = copy.deepcopy(self.calc)
         clqd = copy.deepcopy(self.calc)
         
-        csol['lattice'] = self.lattice.upper()
-        clqd['lattice'] = 'LQD'
-        csol['state'] = 'solid'
-        clqd['state'] = 'liquid'
-        csol['lattice_constant'] = self.lattice_constant
-        clqd['lattice_constant'] = self.lattice_constant
-        csol['thigh'] = self.tmin
-        clqd['thigh'] = 1.5*self.tmax
-        csol['mode'] = 'ts'
-        clqd['mode'] = 'ts'
+        csol.lattice = self.lattice.upper()
+        clqd.lattice = 'LQD'
+        csol.reference_phase = 'solid'
+        clqd.reference_phase = 'liquid'
+        csol.lattice_constant = self.lattice_constant
+        clqd.lattice_constant = self.lattice_constant
+        csol._temperature_high = self.tmin
+        clqd._temperature_high = 1.5*self.tmax
+        csol.mode = 'ts'
+        clqd.mode = 'ts'
         
-        csol['directory'] = create_identifier(csol)
-        clqd['directory'] = create_identifier(clqd)
-        self.options['calculations'] = [csol, clqd]
+        #csol['directory'] = create_identifier(csol)
+        #clqd['directory'] = create_identifier(clqd)
+        self.calculations = [csol, clqd]
         
         
     def get_props(self, elem):
@@ -166,22 +161,24 @@ class MeltingTemp:
 
         self.prepare_calcs()
 
-        self.soljob = cq.setup_calculation(self.options, 0)
-        self.lqdjob = cq.setup_calculation(self.options, 1)
+        self.soljob = Solid(calculation=self.calculations[0], 
+            simfolder=self.calculations[0].create_folders())
+        self.lqdjob = Liquid(calculation=self.calculations[1], 
+            simfolder=self.calculations[1].create_folders())
         
-        self.logger.info("Free energy of %s and %s phases will be calculated"%(self.soljob.calc['lattice'], self.lqdjob.calc['lattice']))
+        self.logger.info("Free energy of %s and %s phases will be calculated"%(self.soljob.calc.lattice, self.lqdjob.calc.lattice))
         self.logger.info("Temperature range of %f-%f"%(self.tmin, self.tmax))
         self.logger.info("STATE: Temperature range of %f-%f K"%(self.tmin, self.tmax))
         self.logger.info('Starting solid fe calculation')
         
         try:
-            self.soljob = cq.routine_fe(self.soljob)
+            self.soljob = routine_fe(self.soljob)
         except MeltedError:
             self.logger.info('Solid phase melted')
             return 2
         
         self.logger.info('Starting solid reversible scaling run')
-        for i in range(self.soljob.nsims):
+        for i in range(self.soljob.calc.n_iterations):
             try:
                 self.soljob.reversible_scaling(iteration=(i+1))
             except MeltedError:
@@ -193,13 +190,13 @@ class MeltingTemp:
         
         self.logger.info('Starting liquid fe calculation')
         try:
-            self.lqdjob = cq.routine_fe(self.lqdjob)
+            self.lqdjob = routine_fe(self.lqdjob)
         except SolidifiedError:
             self.logger.info('Liquid froze')
             return 3
 
         self.logger.info('Starting liquid reversible scaling calculation')
-        for i in range(self.lqdjob.nsims):
+        for i in range(self.lqdjob.calc.n_iterations):
             try:
                 self.lqdjob.reversible_scaling(iteration=(i+1))
             except SolidifiedError:
@@ -239,7 +236,7 @@ class MeltingTemp:
                 return True
             
             self.attempts += 1
-            if (self.attempts>self.maxattempts):
+            if (self.attempts > self.maxattempts):
                 raise ValueError('Maximum number of tries reached')
 
 
@@ -333,4 +330,108 @@ class MeltingTemp:
         self.logger.info('Found melting temperature = %.2f +/- %.2f K '%(tm, tmerr))
         self.logger.info('Experimental melting temperature = %.2f K '%(self.org_tm))
         self.logger.info('STATE: Tm = %.2f K +/- %.2f K, Exp. Tm = %.2f K'%(tm, tmerr, self.org_tm))
-        
\ No newline at end of file
+
+def routine_fe(job):
+    """
+    Perform an FE calculation routine
+    """
+    ts = time.time()
+    job.run_averaging()
+    te = (time.time() - ts)
+    job.logger.info("Averaging routine finished in %f s"%te)
+
+    #now run integration loops
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.run_integration(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("Integration cycle %d finished in %f s"%(i+1, te))
+
+    job.thermodynamic_integration()
+    job.submit_report()
+    return job
+
+def routine_ts(job):
+    """
+    Perform ts routine
+    """
+    routine_fe(job)
+
+    #now do rev scale steps
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.reversible_scaling(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("TS integration cycle %d finished in %f s"%(i+1, te))
+    
+    job.integrate_reversible_scaling(scale_energy=True)
+    return job
+
+
+def routine_only_ts(job):
+    """
+    Perform sweep without free energy calculation
+    """
+    ts = time.time()
+    job.run_averaging()
+    te = (time.time() - ts)
+    job.logger.info("Averaging routine finished in %f s"%te)
+
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.reversible_scaling(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("TS integration cycle %d finished in %f s"%(i+1, te))
+    return job
+
+def routine_tscale(job):
+    """
+    Perform tscale routine
+    """
+    routine_fe(job)
+
+    #now do rev scale steps
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.temperature_scaling(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("Temperature scaling cycle %d finished in %f s"%(i+1, te))
+    
+    job.integrate_reversible_scaling(scale_energy=False)
+    return job
+
+def routine_pscale(job):
+    """
+    Perform pscale routine
+    """
+    routine_fe(job)
+
+    #now do rev scale steps
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.pressure_scaling(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("Pressure scaling cycle %d finished in %f s"%(i+1, te))
+    
+    job.integrate_pressure_scaling()
+    return job
+
+def routine_alchemy(job):
+    """
+    Perform an FE calculation routine
+    """
+    ts = time.time()
+    job.run_averaging()
+    te = (time.time() - ts)
+    job.logger.info("Averaging routine finished in %f s"%te)
+
+    #now run integration loops
+    for i in range(job.calc.n_iterations):
+        ts = time.time()
+        job.run_integration(iteration=(i+1))
+        te = (time.time() - ts)
+        job.logger.info("Alchemy integration cycle %d finished in %f s"%(i+1, te))
+
+    job.thermodynamic_integration()
+    job.submit_report()
+    return job 
\ No newline at end of file
